﻿using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.WindowsMobile.DirectX;
using Microsoft.WindowsMobile.DirectX.Direct3D;
using Microsoft.Samples.MD3DM;
using System.IO;
using System.Xml;
namespace ARDemo
{
    public interface IMeshData : IDisposable
    {
        void Draw();
    }
    //テクスチャを共有するためのメッシュセット
    public class MeshData : IMeshData
    {
        //キーフレームのデータ保持用
        protected Texture[] tex_data = null;
        protected String[] tex_name = null;
        //
        public Texture[] _textures = null;
        private Material[] _materials = null;
        private Mesh m_mesh;
        private Device m_ref_dev;

        public MeshData(Device i_dev, String i_dir, String i_mesh_filename)
        {
            this.m_ref_dev = i_dev;

            // Ｘファイルを読み込んでメッシュを作成する
            this.m_mesh = MeshLoader.LoadMesh(i_dev,
                new FileStream(i_dir + i_mesh_filename, FileMode.Open, FileAccess.Read), MeshFlags.SystemMemory,
                out this._materials, out this.tex_name);
            //テクスチャ/テクスチャ名配列のセット
            this.tex_data = new Texture[tex_name.Length];
            for (int i = 0; i < tex_data.Length; i++)
            {
                // Create the texture
                if (this.tex_name[i] != null)
                {
                    this.tex_data[i] = TextureLoader.FromStream(i_dev, new FileStream(i_dir + this.tex_name[i], FileMode.Open, FileAccess.Read));
                }
            }
            //テクスチャの設定
            this._textures = this.tex_data;
            //マテリアルの設定
            for (int i = 0; i < this._materials.Length; i++)
            {
                if (this._textures[i] != null)
                {
                    //テクスチャの場合は強制的に素材の色を出す。
                    this._materials[i].AmbientColor = new ColorValue(0, 0, 0);
                    this._materials[i].DiffuseColor = new ColorValue(128, 128, 128);
                }
                else
                {
                    this._materials[i].Ambient = this._materials[i].Diffuse;
                    this._materials[i].DiffuseColor = new ColorValue(8, 8, 8);
                }
            }
            return;
        }
        public MeshData(Device i_dev, String i_dir, String i_mesh_filename, MeshData i_keymesh)
        {
            this.m_ref_dev = i_dev;
            string[] tex_names = null;
            // Ｘファイルを読み込んでメッシュを作成する
            this.m_mesh = MeshLoader.LoadMesh(i_dev,
                new FileStream(i_dir + i_mesh_filename, FileMode.Open, FileAccess.Read), MeshFlags.SystemMemory,
                out this._materials, out tex_names);
            //テクスチャの設定
            this._textures = new Texture[tex_names.Length];
            for (int i = 0; i < tex_names.Length; i++)
            {
                if (tex_names[i] == null)
                {
                    continue;
                }
                this._textures[i] = null;
                //テクスチャ名に一致するテクスチャをセット
                for (int j = 0; j < i_keymesh.tex_name.Length; j++)
                {
                    if (tex_names[i] == i_keymesh.tex_name[j])
                    {
                        this._textures[i] = i_keymesh.tex_data[j];
                        break;
                    }
                }
            }
            for (int i = 0; i < this._materials.Length; i++)
            {
                if (this._textures[i] != null)
                {
                    //テクスチャの場合は強制的に素材の色を出す。
                    this._materials[i].AmbientColor = new ColorValue(0, 0, 0);
                    this._materials[i].DiffuseColor = new ColorValue(128, 128, 128);
                }
                else
                {
                    this._materials[i].Ambient = this._materials[i].Diffuse;
                    this._materials[i].DiffuseColor = new ColorValue(8, 8, 8);
                }
            }
            return;
        }
        public void Draw()
        {
            for (int i = 0; i < this._materials.Length; i++)
            {
                // マテリアルをセット
                this.m_ref_dev.Material = this._materials[i];
                // テクスチャーのセット
                this.m_ref_dev.SetTexture(0, this._textures[i]);
                // 描画
                this.m_mesh.DrawSubset(i);
            }
            return;
        }
        public void Dispose()
        {
            if (this.tex_data != null)
            {
                for (int i = 0; i < tex_data.Length; i++)
                {
                    if (this.tex_data[i] != null)
                    {
                        this.tex_data[i].Dispose();
                    }
                }
            }
            return;
        }
    }

    public class MeshSet : IDisposable
    {
        private uint _draw_counter = 0;
        private IMeshData[] _mesh_data;
        public MeshSet(D3dManager i_d3dmgr, String i_xml_path)
        {
            Device dev = i_d3dmgr.d3d_device;
            XmlDocument dom = new XmlDocument();
            //ファイル名の確定
            dom.Load(i_xml_path);

            if (dom.SelectSingleNode("/root/version").InnerText != "ARDemo/0.1;MeshSet")
            {
                throw new Exception("object::設定ファイルのバージョンが違います？");
            }
            //バカだからタイプを確認しない

            //スケール取得
            float sc = float.Parse(dom.SelectSingleNode("/root/md3dmset/@scale").Value);
            this._scale = Matrix.Scaling(sc, sc, sc);

            //md3dmsetからメッシュリストを作る。
            XmlNodeList md3dm_nodes = dom.SelectNodes("/root/md3dmset/md3dm/@file");
            if (md3dm_nodes.Count < 1)
            {
                throw new Exception("MeshSet:has not md3dm node.");
            }
            this._mesh_data = new IMeshData[md3dm_nodes.Count];
            //xmlの有るパスを取得
            String md3dm_root_path = System.IO.Path.GetDirectoryName(i_xml_path);
            //
            MeshData zero = new MeshData(dev, md3dm_root_path + "\\", md3dm_nodes[0].Value);
            this._mesh_data[0] = zero;
            for (int i = 1; i < md3dm_nodes.Count; i++)
            {
                this._mesh_data[i] = new MeshData(dev, md3dm_root_path + "\\", md3dm_nodes[i].Value, zero);
            }
            return;
        }

        private Matrix _scale;
        public Matrix scale { get { return this._scale; } }

        public void Draw()
        {
            this._mesh_data[this._draw_counter % this._mesh_data.Length].Draw();
            this._draw_counter++;
            return;
        }
        public void Dispose()
        {
            for (int i = 0; i < this._mesh_data.Length - 1; i++)
            {
                if (this._mesh_data[i] != null)
                {
                    this._mesh_data[i].Dispose();
                }
            }
            return;
        }
    }

}
